@@ -44,8 +44,6 @@
 
 #include <private/qanimationgroupjob_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class Q_QML_PRIVATE_EXPORT QPauseAnimationJob : public QAbstractAnimationJob
@@ -60,6 +58,7 @@ public:
 
 protected:
     void updateCurrentTime(int);
+    void debugAnimation(QDebug d) const;
 
 private:
     //definition
@@ -68,6 +67,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QPAUSEANIMATIONJOB_P_H
