@@ -88,6 +88,7 @@ QQuickTextPrivate::QQuickTextPrivate()
     , truncated(false), hAlignImplicit(true), rightToLeftText(false)
     , layoutTextElided(false), textHasChanged(true), needToUpdateLayout(false), formatModifiesFontSize(false)
 {
+    implicitAntialiasing = true;
 }
 
 QQuickTextPrivate::ExtraData::ExtraData()
@@ -114,6 +115,7 @@ QQuickTextDocumentWithImageResources::QQuickTextDocumentWithImageResources(QQuic
 {
     setUndoRedoEnabled(false);
     documentLayout()->registerHandler(QTextFormat::ImageObject, this);
+    connect(this, SIGNAL(baseUrlChanged(QUrl)), this, SLOT(reset()));
 }
 
 QQuickTextDocumentWithImageResources::~QQuickTextDocumentWithImageResources()
@@ -125,14 +127,13 @@ QQuickTextDocumentWithImageResources::~QQuickTextDocumentWithImageResources()
 QVariant QQuickTextDocumentWithImageResources::loadResource(int type, const QUrl &name)
 {
     QQmlContext *context = qmlContext(parent());
-    QUrl url = m_baseUrl.resolved(name);
 
     if (type == QTextDocument::ImageResource) {
-        QQuickPixmap *p = loadPixmap(context, url);
+        QQuickPixmap *p = loadPixmap(context, name);
         return p->image();
     }
 
-    return QTextDocument::loadResource(type,url); // The *resolved* URL
+    return QTextDocument::loadResource(type, name);
 }
 
 void QQuickTextDocumentWithImageResources::requestFinished()
@@ -144,14 +145,6 @@ void QQuickTextDocumentWithImageResources::requestFinished()
     }
 }
 
-void QQuickTextDocumentWithImageResources::clear()
-{
-    clearResources();
-
-    QTextDocument::clear();
-}
-
-
 QSizeF QQuickTextDocumentWithImageResources::intrinsicSize(
         QTextDocument *, int, const QTextFormat &format)
 {
@@ -166,7 +159,7 @@ QSizeF QQuickTextDocumentWithImageResources::intrinsicSize(
         QSizeF size(width, height);
         if (!hasWidth || !hasHeight) {
             QQmlContext *context = qmlContext(parent());
-            QUrl url = m_baseUrl.resolved(QUrl(imageFormat.name()));
+            QUrl url = baseUrl().resolved(QUrl(imageFormat.name()));
 
             QQuickPixmap *p = loadPixmap(context, url);
             if (!p->isReady()) {
@@ -204,19 +197,16 @@ void QQuickTextDocumentWithImageResources::drawObject(
 QImage QQuickTextDocumentWithImageResources::image(const QTextImageFormat &format)
 {
     QQmlContext *context = qmlContext(parent());
-    QUrl url = m_baseUrl.resolved(QUrl(format.name()));
+    QUrl url = baseUrl().resolved(QUrl(format.name()));
 
     QQuickPixmap *p = loadPixmap(context, url);
     return p->image();
 }
 
-void QQuickTextDocumentWithImageResources::setBaseUrl(const QUrl &url, bool clear)
+void QQuickTextDocumentWithImageResources::reset()
 {
-    m_baseUrl = url;
-    if (clear) {
-        clearResources();
-        markContentsDirty(0, characterCount());
-    }
+    clearResources();
+    markContentsDirty(0, characterCount());
 }
 
 QQuickPixmap *QQuickTextDocumentWithImageResources::loadPixmap(
@@ -297,6 +287,15 @@ qreal QQuickTextPrivate::getImplicitHeight() const
     return implicitHeight;
 }
 
+/*!
+    \qmlproperty bool QtQuick::Text::antialiasing
+
+    Used to decide if the Text should use antialiasing or not. Only Text
+    with renderType of Text.NativeRendering can disable antialiasing.
+
+    The default is true.
+*/
+
 void QQuickText::q_imagesLoaded()
 {
     Q_D(QQuickText);
@@ -620,8 +619,9 @@ void QQuickTextPrivate::setupCustomLineGeometry(QTextLine &line, qreal &height,
     textLine->setHeight(0);
     textLine->setLineOffset(lineOffset);
 
-    // use the text item's width by default if it has one and wrap is on
-    if (q->widthValid() && q->wrapMode() != QQuickText::NoWrap)
+    // use the text item's width by default if it has one and wrap is on or text must be aligned
+    if (q->widthValid() && (q->wrapMode() != QQuickText::NoWrap ||
+                            q->effectiveHAlign() != QQuickText::AlignLeft))
         textLine->setWidth(q->width());
     else
         textLine->setWidth(INT_MAX);
@@ -959,7 +959,7 @@ QRectF QQuickTextPrivate::setupTextLayout(qreal *const baseline)
 
             // If the horizontal alignment is not left and the width was not valid we need to relayout
             // now that we know the maximum line width.
-            if (!implicitWidthValid && lineCount > 1 && q->effectiveHAlign() != QQuickText::AlignLeft) {
+            if (!implicitWidthValid && unwrappedLineCount > 1 && q->effectiveHAlign() != QQuickText::AlignLeft) {
                 widthExceeded = false;
                 heightExceeded = false;
                 continue;
@@ -1250,9 +1250,9 @@ QQuickText::~QQuickText()
 */
 
 /*!
-    \qmlsignal QtQuick::Text::onLineLaidOut(object line)
+    \qmlsignal QtQuick::Text::lineLaidOut(object line)
 
-    This handler is called for each line of text that is laid out during the layout
+    This signal is emitted for each line of text that is laid out during the layout
     process. The specified \a line object provides more details about the line that
     is currently being laid out.
 
@@ -1277,12 +1277,14 @@ QQuickText::~QQuickText()
         }
     }
     \endcode
+
+    The corresponding handler is \c onLineLaidOut.
 */
 
 /*!
-    \qmlsignal QtQuick::Text::onLinkActivated(string link)
+    \qmlsignal QtQuick::Text::linkActivated(string link)
 
-    This handler is called when the user clicks on a link embedded in the text.
+    This signal is emitted when the user clicks on a link embedded in the text.
     The link must be in rich text or HTML format and the
     \a link string provides access to the particular link.
 
@@ -1293,6 +1295,8 @@ QQuickText::~QQuickText()
 
     Clicking on the highlighted link will output
     \tt{http://qt-project.org link activated} to the console.
+
+    The corresponding handler is \c onLinkActivated.
 */
 
 /*!
@@ -1415,6 +1419,9 @@ void QQuickText::setFont(const QFont &font)
     QFont oldFont = d->font;
     d->font = font;
 
+    if (!antialiasing())
+        d->font.setStyleStrategy(QFont::NoAntialias);
+
     if (d->font.pointSizeF() != -1) {
         // 0.5pt resolution
         qreal size = qRound(d->font.pointSizeF()*2.0);
@@ -1434,6 +1441,21 @@ void QQuickText::setFont(const QFont &font)
     emit fontChanged(d->sourceFont);
 }
 
+void QQuickText::itemChange(ItemChange change, const ItemChangeData &value)
+{
+    Q_D(QQuickText);
+    Q_UNUSED(value);
+    if (change == ItemAntialiasingHasChanged) {
+        if (!antialiasing())
+            d->font.setStyleStrategy(QFont::NoAntialias);
+        else
+            d->font.setStyleStrategy(QFont::PreferAntialias);
+        d->implicitWidthValid = false;
+        d->implicitHeightValid = false;
+        d->updateLayout();
+    }
+}
+
 /*!
     \qmlproperty string QtQuick::Text::text
 
@@ -2070,7 +2092,7 @@ QRectF QQuickText::boundingRect() const
     Q_D(const QQuickText);
 
     QRectF rect = d->layedOutTextRect;
-    rect.moveLeft(QQuickTextUtil::alignedX(rect.width(), width(), d->hAlign));
+    rect.moveLeft(QQuickTextUtil::alignedX(rect.width(), width(), effectiveHAlign()));
     rect.moveTop(QQuickTextUtil::alignedY(rect.height(), height(), d->vAlign));
 
     if (d->style != Normal)
@@ -2198,7 +2220,7 @@ QSGNode *QQuickText::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *data
     else
         node = static_cast<QQuickTextNode *>(oldNode);
 
-    node->setUseNativeRenderer(d->renderType == NativeRendering && d->window->devicePixelRatio() <= 1);
+    node->setUseNativeRenderer(d->renderType == NativeRendering);
     node->deleteContent();
     node->setMatrix(QMatrix4x4());
 
@@ -2207,11 +2229,11 @@ QSGNode *QQuickText::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeData *data
     const QColor linkColor = QColor::fromRgba(d->linkColor);
 
     if (d->richText) {
-        const qreal dx = QQuickTextUtil::alignedX(d->layedOutTextRect.width(), width(), d->hAlign);
+        const qreal dx = QQuickTextUtil::alignedX(d->layedOutTextRect.width(), width(), effectiveHAlign());
         d->ensureDoc();
         node->addTextDocument(QPointF(dx, dy), d->extra->doc, color, d->style, styleColor, linkColor);
     } else if (d->layedOutTextRect.width() > 0) {
-        const qreal dx = QQuickTextUtil::alignedX(d->lineWidth, width(), d->hAlign);
+        const qreal dx = QQuickTextUtil::alignedX(d->lineWidth, width(), effectiveHAlign());
         int unelidedLineCount = d->lineCount;
         if (d->elideLayout)
             unelidedLineCount -= 1;
@@ -2337,7 +2359,7 @@ void QQuickText::setLineHeightMode(LineHeightMode mode)
     within the width of the item without wrapping is used.
     \li Text.VerticalFit - The largest size up to the size specified that fits
     the height of the item is used.
-    \li Text.Fit - The largest size up to the size specified the fits within the
+    \li Text.Fit - The largest size up to the size specified that fits within the
     width and height of the item is used.
     \endlist
 
@@ -2345,6 +2367,10 @@ void QQuickText::setLineHeightMode(LineHeightMode mode)
     minimumPointSize or minimumPixelSize property and maximum bound specified
     by either the \l font.pointSize or \l font.pixelSize properties.
 
+    \qml
+    Text { text: "Hello"; fontSizeMode: Text.Fit; minimumPixelSize: 10; font.pixelSize: 72 }
+    \endqml
+
     If the text does not fit within the item bounds with the minimum font size
     the text will be elided as per the \l elide property.
 */
@@ -2483,7 +2509,7 @@ QString QQuickTextPrivate::anchorAt(const QPointF &mousePos) const
             link = anchorAt(elideLayout, translatedMousePos);
         return link;
     } else if (richText && extra.isAllocated() && extra->doc) {
-        translatedMousePos.rx() -= QQuickTextUtil::alignedX(layedOutTextRect.width(), q->width(), hAlign);
+        translatedMousePos.rx() -= QQuickTextUtil::alignedX(layedOutTextRect.width(), q->width(), q->effectiveHAlign());
         return extra->doc->documentLayout()->anchorAt(translatedMousePos);
     }
     return QString();
@@ -2544,14 +2570,16 @@ bool QQuickTextPrivate::isLinkHoveredConnected()
 }
 
 /*!
-    \qmlsignal QtQuick::Text::onLinkHovered(string link)
+    \qmlsignal QtQuick::Text::linkHovered(string link)
     \since 5.2
 
-    This handler is called when the user hovers a link embedded in the
+    This signal is emitted when the user hovers a link embedded in the
     text. The link must be in rich text or HTML format and the \a link
     string provides access to the particular link.
 
-    \sa hoveredLink
+    The corresponding handler is \c onLinkHovered.
+
+    \sa hoveredLink, linkAt()
 */
 
 /*!
@@ -2562,7 +2590,7 @@ bool QQuickTextPrivate::isLinkHoveredConnected()
     embedded in the text. The link must be in rich text or HTML format
     and the \a hoveredLink string provides access to the particular link.
 
-    \sa onLinkHovered
+    \sa linkHovered, linkAt()
 */
 
 QString QQuickText::hoveredLink() const
@@ -2631,9 +2659,6 @@ void QQuickText::hoverLeaveEvent(QHoverEvent *event)
     not require advanced features such as transformation of the text. Using such features in
     combination with the NativeRendering render type will lend poor and sometimes pixelated
     results.
-
-    On HighDpi "retina" displays and mobile and embedded platforms, this property is ignored
-    and QtRendering is always used.
 */
 QQuickText::RenderType QQuickText::renderType() const
 {
@@ -2665,4 +2690,19 @@ void QQuickText::doLayout()
     d->updateSize();
 }
 
+/*!
+    \qmlmethod QtQuick::Text::linkAt(real x, real y)
+    \since 5.3
+
+    Returns the link string at point \a x, \a y in content coordinates,
+    or an empty string if no link exists at that point.
+
+    \sa hoveredLink
+*/
+QString QQuickText::linkAt(qreal x, qreal y) const
+{
+    Q_D(const QQuickText);
+    return d->anchorAt(QPointF(x, y));
+}
+
 QT_END_NAMESPACE
