@@ -132,6 +132,10 @@ public:
     void setClearMode(ClearMode mode) { m_clear_mode = mode; }
     ClearMode clearMode() const { return m_clear_mode; }
 
+    virtual void setCustomRenderMode(const QByteArray &) { };
+
+    void clearChangedFlag() { m_changed_emitted = false; }
+
 Q_SIGNALS:
     void sceneGraphChanged(); // Add, remove, ChangeFlags changes...
 
@@ -148,6 +152,7 @@ protected:
     void addNodesToPreprocess(QSGNode *node);
     void removeNodesToPreprocess(QSGNode *node);
 
+    void markNodeDirtyState(QSGNode *node, QSGNode::DirtyState state) { node->m_dirtyState |= state; }
 
     QColor m_clear_color;
     ClearMode m_clear_mode;
