@@ -52,7 +52,7 @@
 
 #include <private/qsgtexture_p.h>
 
-#include <private/qqmlprofilerservice_p.h>
+#include <private/qquickprofiler_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -140,20 +140,26 @@ Atlas::Atlas(const QSize &size)
     : m_allocator(size)
     , m_texture_id(0)
     , m_size(size)
-    , m_filtering(QSGTexture::Linear)
     , m_allocated(false)
 {
 
-#ifdef QT_OPENGL_ES
+    m_internalFormat = GL_RGBA;
+    m_externalFormat = GL_BGRA;
+
+#ifndef QT_OPENGL_ES
+    if (QOpenGLContext::currentContext()->isOpenGLES()) {
+#endif
+
 #if defined(Q_OS_ANDROID) && !defined(Q_OS_ANDROID_NO_SDK)
     QString *deviceName =
             static_cast<QString *>(QGuiApplication::platformNativeInterface()->nativeResourceForIntegration("AndroidDeviceName"));
-    static bool wrongfullyReportsBgra8888Support = deviceName->compare(QStringLiteral("samsung SM-T211"), Qt::CaseInsensitive) == 0
-                                                || deviceName->compare(QStringLiteral("samsung SM-T210"), Qt::CaseInsensitive) == 0
-                                                || deviceName->compare(QStringLiteral("samsung SM-T215"), Qt::CaseInsensitive) == 0;
+    static bool wrongfullyReportsBgra8888Support = deviceName != 0
+                                                    && (deviceName->compare(QStringLiteral("samsung SM-T211"), Qt::CaseInsensitive) == 0
+                                                        || deviceName->compare(QStringLiteral("samsung SM-T210"), Qt::CaseInsensitive) == 0
+                                                        || deviceName->compare(QStringLiteral("samsung SM-T215"), Qt::CaseInsensitive) == 0);
 #else
     static bool wrongfullyReportsBgra8888Support = false;
-#endif
+#endif // ANDROID
 
     const char *ext = (const char *) glGetString(GL_EXTENSIONS);
     if (!wrongfullyReportsBgra8888Support
@@ -165,13 +171,13 @@ Atlas::Atlas(const QSize &size)
     } else if (strstr(ext, "GL_APPLE_texture_format_BGRA8888")) {
         m_internalFormat = GL_RGBA;
         m_externalFormat = GL_BGRA;
-#endif
+#endif // IOS
     } else {
         m_internalFormat = m_externalFormat = GL_RGBA;
     }
-#else
-    m_internalFormat = GL_RGBA;
-    m_externalFormat = GL_BGRA;
+
+#ifndef QT_OPENGL_ES
+    }
 #endif
 
     m_use_bgra_fallback = qEnvironmentVariableIsSet("QSG_ATLAS_USE_BGRA_FALLBACK");
@@ -269,7 +275,7 @@ void Atlas::uploadBgra(Texture *texture)
     QImage image = texture->image();
 
     if (image.format() != QImage::Format_ARGB32_Premultiplied
-            || image.format() != QImage::Format_RGB32) {
+            && image.format() != QImage::Format_RGB32) {
         image = image.convertToFormat(QImage::Format_ARGB32_Premultiplied);
     }
 
@@ -314,9 +320,8 @@ void Atlas::uploadBgra(Texture *texture)
 
 }
 
-bool Atlas::bind(QSGTexture::Filtering filtering)
+void Atlas::bind(QSGTexture::Filtering filtering)
 {
-    bool forceUpdate = false;
     if (!m_allocated) {
         m_allocated = true;
 
@@ -329,7 +334,8 @@ bool Atlas::bind(QSGTexture::Filtering filtering)
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 #if !defined(QT_OPENGL_ES_2)
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
+        if (!QOpenGLContext::currentContext()->isOpenGLES())
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_LEVEL, 0);
 #endif
         glTexImage2D(GL_TEXTURE_2D, 0, m_internalFormat, m_size.width(), m_size.height(), 0, m_externalFormat, GL_UNSIGNED_BYTE, 0);
 
@@ -361,19 +367,18 @@ bool Atlas::bind(QSGTexture::Filtering filtering)
             glDeleteTextures(1, &m_texture_id);
             m_texture_id = 0;
         }
-        forceUpdate = true;
     } else {
         glBindTexture(GL_TEXTURE_2D, m_texture_id);
     }
 
     if (m_texture_id == 0)
-        return false;
+        return;
 
     // Upload all pending images..
     for (int i=0; i<m_pending_uploads.size(); ++i) {
 
 #ifndef QSG_NO_RENDER_TIMING
-        bool profileFrames = qsg_render_timing || QQmlProfilerService::enabled;
+        bool profileFrames = qsg_render_timing || QQuickProfiler::enabled;
         if (profileFrames)
             qsg_renderer_timer.start();
 #endif
@@ -393,28 +398,20 @@ bool Atlas::bind(QSGTexture::Filtering filtering)
                    (int) (qsg_renderer_timer.elapsed()));
         }
 
-        if (QQmlProfilerService::enabled) {
-            QQmlProfilerService::sceneGraphFrame(
-                        QQmlProfilerService::SceneGraphTexturePrepare,
-                        0,  // bind (not relevant)
-                        0,  // convert (not relevant)
-                        0,  // swizzle (not relevant)
-                        qsg_renderer_timer.nsecsElapsed(), // (upload all of the above)
-                        0); // mipmap (not used ever...)
-        }
+        Q_QUICK_SG_PROFILE1(QQuickProfiler::SceneGraphTexturePrepare, (
+                0,  // bind (not relevant)
+                0,  // convert (not relevant)
+                0,  // swizzle (not relevant)
+                qsg_renderer_timer.nsecsElapsed(), // (upload all of the above)
+                0)); // mipmap (not used ever...)
 #endif
     }
 
-    if (filtering != m_filtering) {
-        GLenum f = filtering == QSGTexture::Nearest ? GL_NEAREST : GL_LINEAR;
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, f);
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, f);
-        m_filtering = filtering;
-    }
+    GLenum f = filtering == QSGTexture::Nearest ? GL_NEAREST : GL_LINEAR;
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, f);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, f);
 
     m_pending_uploads.clear();
-
-    return forceUpdate;
 }
 
 void Atlas::remove(Texture *t)
@@ -461,6 +458,7 @@ QSGTexture *Texture::removedFromAtlas() const
         m_nonatlas_texture = new QSGPlainTexture();
         m_nonatlas_texture->setImage(m_image);
         m_nonatlas_texture->setFiltering(filtering());
+        m_nonatlas_texture->setMipmapFiltering(mipmapFiltering());
     }
     return m_nonatlas_texture;
 }
