@@ -200,19 +200,21 @@ QQmlProperty QQuickAbstractAnimationPrivate::createProperty(QObject *obj, const
 }
 
 /*!
-    \qmlsignal QtQuick::Animation::onStarted()
+    \qmlsignal QtQuick::Animation::started()
 
-    This signal handler is called when the animation begins.
+    This signal is emitted when the animation begins.
 
     It is only triggered for top-level, standalone animations. It will not be
     triggered for animations in a Behavior or Transition, or animations
     that are part of an animation group.
+
+    The corresponding handler is \c onStarted.
 */
 
 /*!
-    \qmlsignal QtQuick::Animation::onStopped()
+    \qmlsignal QtQuick::Animation::stopped()
 
-    This signal handler is called when the animation ends.
+    This signal is emitted when the animation ends.
 
     The animation may have been stopped manually, or may have run to completion.
 
@@ -220,8 +222,10 @@ QQmlProperty QQuickAbstractAnimationPrivate::createProperty(QObject *obj, const
     triggered for animations in a Behavior or Transition, or animations
     that are part of an animation group.
 
-    If \l alwaysRunToEnd is true, onStopped will not be called until the animation
+    If \l alwaysRunToEnd is true, this signal will not be emitted until the animation
     has completed its current iteration.
+
+    The corresponding handler is \c onStopped.
 */
 
 void QQuickAbstractAnimation::setRunning(bool r)
@@ -869,6 +873,19 @@ void QActionAnimation::updateState(State newState, State oldState)
     }
 }
 
+void QActionAnimation::debugAnimation(QDebug d) const
+{
+    d << "ActionAnimation(" << hex << (void *) this << dec << ")";
+
+    if (animAction) {
+        int indentLevel = 1;
+        const QAbstractAnimationJob *job = this;
+        while ((job = job->group()))
+            ++indentLevel;
+        animAction->debugAction(d, indentLevel);
+    }
+}
+
 /*!
     \qmltype ScriptAction
     \instantiates QQuickScriptAction
@@ -953,6 +970,22 @@ QAbstractAnimationAction* QQuickScriptActionPrivate::createAction()
     return new Proxy(this);
 }
 
+void QQuickScriptActionPrivate::debugAction(QDebug d, int indentLevel) const
+{
+    QQmlScriptString scriptStr = hasRunScriptScript ? runScriptScript : script;
+
+    if (!scriptStr.isEmpty()) {
+        QQmlExpression expr(scriptStr);
+
+        QByteArray ind(indentLevel, ' ');
+        QString exprStr = expr.expression();
+        int endOfFirstLine = exprStr.indexOf('\n');
+        d << "\n" << ind.constData() << exprStr.left(endOfFirstLine);
+        if (endOfFirstLine != -1 && endOfFirstLine < exprStr.length())
+            d << "...";
+    }
+}
+
 void QQuickScriptActionPrivate::execute()
 {
     Q_Q(QQuickScriptAction);
@@ -1166,6 +1199,14 @@ QAbstractAnimationJob* QQuickPropertyAction::transition(QQuickStateActions &acti
                 QQmlPropertyPrivate::write(action.property, action.toValue, QQmlPropertyPrivate::BypassInterceptor | QQmlPropertyPrivate::DontRemoveBinding);
             }
         }
+        virtual void debugAction(QDebug d, int indentLevel) const {
+            QByteArray ind(indentLevel, ' ');
+            for (int ii = 0; ii < actions.count(); ++ii) {
+                const QQuickStateAction &action = actions.at(ii);
+                d << "\n" << ind.constData() << "target:" << action.property.object() << "property:" << action.property.name()
+                  << "value:" << action.toValue;
+            }
+        }
     };
 
     QStringList props = d->properties.isEmpty() ? QStringList() : d->properties.split(QLatin1Char(','));
@@ -1925,6 +1966,19 @@ void QQuickBulkValueAnimator::topLevelAnimationLoopChanged()
     QAbstractAnimationJob::topLevelAnimationLoopChanged();
 }
 
+void QQuickBulkValueAnimator::debugAnimation(QDebug d) const
+{
+    d << "BulkValueAnimation(" << hex << (void *) this << dec << ")" << "duration:" << duration();
+
+    if (animValue) {
+        int indentLevel = 1;
+        const QAbstractAnimationJob *job = this;
+        while ((job = job->group()))
+            ++indentLevel;
+        animValue->debugUpdater(d, indentLevel);
+    }
+}
+
 /*!
     \qmltype PropertyAnimation
     \instantiates QQuickPropertyAnimation
@@ -2498,6 +2552,16 @@ void QQuickAnimationPropertyUpdater::setValue(qreal v)
     fromSourced = true;
 }
 
+void QQuickAnimationPropertyUpdater::debugUpdater(QDebug d, int indentLevel) const
+{
+    QByteArray ind(indentLevel, ' ');
+    for (int i = 0; i < actions.count(); ++i) {
+        const QQuickStateAction &action = actions.at(i);
+        d << "\n" << ind.constData() << "target:" << action.property.object() << "property:" << action.property.name()
+          << "from:" << action.fromValue << "to:" << action.toValue;
+    }
+}
+
 QQuickStateActions QQuickPropertyAnimation::createTransitionActions(QQuickStateActions &actions,
                                                                                 QQmlProperties &modified,
                                                                                 QObject *defaultTarget)
